import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import usefulfunctions as uf
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.legend_handler import HandlerTuple
import matplotlib.gridspec as gridspec


# plotting parameters
plt.rc('font', family='serif', serif='Arial')
single_col_width = 3.26772
double_col_width = 6.73228
fs = 8
lw = 1
ax_lw = 0.8
ax_tickl = 4
cut = 10
cmap = plt.cm.RdBu_r
ms = 2.5
cbar_ax_colour = 'k'
colour_rubrene = 'darkorange'
colour_dbp = 'darkred'
colour_triplet = 'darkblue'
dpi = 300
fmt = 'svg'



# TA data
ids = ['NP_3', 'solution', 'film']
ta, kn, sp = {}, {}, {}
for idx in ids:
    
    df = pd.read_csv(os.path.join('data', 'Rub_{0}_TA_data.csv'.format(idx)), header=0, index_col=0)
    df.columns = [float(x) for x in df.columns]
    ta[idx] = df
    
    df = pd.read_csv(os.path.join('data', 'Rub_{0}_TA_spectra.csv'.format(idx)), header=0, index_col=0)
    sp[idx] = df
    
    df = pd.read_csv(os.path.join('data', 'Rub_{0}_TA_dynamics.csv'.format(idx)), header=0, index_col=0)
    kn[idx] = df
    
ta_labels = dict(zip(ids, ['(i) nanoparticle film', '(ii) dilute solution', '(iii) polycrystalline film']))


df = pd.read_csv(os.path.join('data', 'Rub_NP_1_TA_dynamics.csv'), header=0, index_col=0)
kn['NP_1'] = df
df = pd.read_csv(os.path.join('data', 'Rub_NP_2_TA_dynamics.csv'), header=0, index_col=0)
kn['NP_2'] = df


#ref_triplet_ESA = pd.read_csv(os.path.join('data', 'Yee1979_T.csv'), header=None, index_col=0, squeeze=True)
#ref_singlet_ESA = pd.read_csv(os.path.join('data', 'Lohmannsroben1988_S.csv'), header=None, index_col=0, squeeze=True)




# create the grid
fig = plt.figure()

grid = gridspec.GridSpec(2, 1, hspace=0.22, height_ratios=[2.5, 1])

gridrt = gridspec.GridSpecFromSubplotSpec(2, 3, subplot_spec=grid[0], wspace=0.07, hspace=0.07, height_ratios=[1.5, 1])
gridrb = gridspec.GridSpecFromSubplotSpec(1, 3, subplot_spec=grid[1], wspace=0.07)

# ps TA data
for i, idx in enumerate(ids):
    
    ta_x = ta[idx]
    sp_x = sp[idx]
    kn_x = kn[idx]
    
    # colour maps
    
    inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridrt[0, i])
    ax = plt.Subplot(fig, inner[0])
    fig.add_subplot(ax)
    
    x = ta_x.index.values
    y = ta_x.columns.values
    X, Y = np.meshgrid(x, y)
    Z = np.transpose(ta_x.values*1000)
    zmax = np.nanmax(np.abs(Z))
    levels = np.linspace(-zmax, zmax, 50)
    
    cs = ax.contourf(X, Y, Z, levels, cmap=cmap, extend='both', zorder=-20)
    for c in cs.collections:
        c.set_edgecolor('face')
    ax.set_rasterization_zorder(-10)
    ax.set_xlim([min(x), max(x)])
    ax.set_ylim([0.1, max(y)])
    

    axins = inset_axes(ax, width='5%', height='25%', loc='upper right')
    cbar = fig.colorbar(cs, cax=axins, orientation='vertical', ticks=[-np.floor(zmax), 0, np.floor(zmax)], extendrect=True)
    axins.tick_params(color=cbar_ax_colour, labelcolor=cbar_ax_colour, labelsize=fs)
    axins.yaxis.tick_left()
    axins.text(1.2, -0.1, r'$\Delta$A (mOD)', fontsize=fs, color=cbar_ax_colour, ha='right', va='top', transform=axins.transAxes)

    ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)
    
    ax.set_yscale('log')
    
    ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10)
    ax = uf.logticks(ax, which='y')

    if i > 0:
        ax.set_yticklabels([])
    else:
        ax.set_ylabel('Time (ps)', fontsize=fs)
    
    ax.set_title(ta_labels[idx], fontsize=fs)

    # spectra

    inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridrt[1, i])
    ax = plt.Subplot(fig, inner[0])
    fig.add_subplot(ax)
    
    ax.axhline(0, linewidth=1, color='0.5')
    if i == 1:
        ax.plot(sp_x.index, sp_x/sp_x.max(), color=colour_rubrene, linewidth=lw, label='singlet')
    else:
        ax.plot(sp_x.index, sp_x['S']/sp_x['S'].max(), color=colour_rubrene, linewidth=lw, label='singlet')
        ax.plot(sp_x.index, sp_x['T']/sp_x['T'].max(), color=colour_triplet, linewidth=lw, label='triplet')
        
    if i == 0:
        ref_scale = 18
        #ax.plot(ref_singlet_ESA.index, ref_singlet_ESA/ref_singlet_ESA.max(), 's', color=colour_rubrene, mfc='None', ms=ms+0.5, mew=lw, label='Ref. XX')
        #ax.plot(ref_triplet_ESA.index, ref_scale*ref_triplet_ESA, 'o', color=colour_triplet, mfc='None', ms=ms+0.5, mew=lw, label='Ref. 69')    
        
    ax.set_xlim(sp_x.index.min(), sp_x.index.max())
    ax.set_ylim(-0.1, 1.1)
    
    if i > 0:
        ax.set_yticklabels([])
    else:
        ax.set_ylabel(r'$\Delta$A (norm.)', fontsize=fs)
        
    ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)
    ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=0.2, yminorsep=0.1)
    
    ax.set_xlabel('Wavelength (nm)', fontsize=fs)
    
    ax.legend(frameon=False, fontsize=fs, borderpad=0, loc='upper right', handlelength=1.5)
    
    # kinetics
    
    inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridrb[0, i])
    ax1 = plt.Subplot(fig, inner[0])
    fig.add_subplot(ax1)
    
    if i == 1:
        normf = kn_x[(kn_x.index > 100) & (kn_x.index < 1000)].mean()
        
        x = np.logspace(0, 4, 100)
        ax1.plot(x, np.exp(-x/16000), 'k--', linewidth=lw, label=r'$\tau=16$ns')
        
        ax1.plot(kn_x.index, kn_x/normf, color=colour_rubrene, linestyle='None', marker='o', ms=ms, mew=0, alpha=0.8)
        
        ax1.legend(frameon=False, fontsize=fs, handlelength=1.3, handletextpad=0.3, loc=(0.2, 0.6))
        
    elif i == 2:
        normfs = kn_x['S'].max()
        tscale = 1
        ax1.plot(kn_x.index, kn_x['S']/normfs, color=colour_rubrene, linestyle='None', marker='o', ms=ms, mew=0, alpha=0.8)
        ax1.plot(kn_x.index, tscale*kn_x['T']/normfs, color=colour_triplet, linestyle='None', marker='o', ms=ms, mew=0, alpha=0.8)
        
    elif i == 0:
        alphas = [0.2, 0.4, 0.8]
        conv = 1000/(5000*np.pi*0.05**2)
        powers = conv*np.array([0.3, 0.7, 1.6])
        for p, idx in enumerate(['NP_1', 'NP_2', 'NP_3']):
            kn_x = kn[idx]
            normfs = kn_x['S'].max()
            tscale = 3.5
            ax1.plot(kn_x.index, kn_x['S']/normfs, color=colour_rubrene, linestyle='None', marker='o', ms=ms, mew=0, alpha=alphas[p], label=r'{0:.0f}$\mu$Jcm$^{{-2}}$'.format(powers[p]))
            ax1.plot(kn_x.index, tscale*kn_x['T']/normfs, color=colour_triplet, linestyle='None', marker='o', ms=ms, mew=0, alpha=alphas[p], label=r'{0:.0f}$\mu$Jcm$^{{-2}}$'.format(powers[p]))
            
        ax1.text(2.7, 0.07, r'triplet $\times${0}'.format(tscale), fontsize=fs, color=colour_triplet)
            
        hs, ls = ax1.get_legend_handles_labels()
        #ax1.legend(handles=zip(hs[::2], hs[1::2]), labels=ls[::2], numpoints=1, handler_map={tuple: HandlerTuple(ndivide=None)}, frameon=False, loc=(0, 0.3), borderpad=0, handlelength=1.3, labelspacing=0.08, fontsize=fs, handletextpad=0.1)
        
    ax1.set_xlim([0.1, kn_x.index.max()])
    ax1.set_ylim(-0.1, 1.1)
    ax1.set_xscale('log')
    
    ax1.axhline(0, linewidth=1, color='0.5')   
    
    ax1 = uf.linearticks(ax1, ymajorsep=0.2, yminorsep=0.1)
    ax1 = uf.logticks(ax1, which='x')

    ax1 = uf.format_axes(ax1, labelsize=fs, linewidth=ax_lw, length=ax_tickl)
    
    if i > 0:
        ax1.set_yticklabels([])
    else:
        ax1.set_ylabel(r'$\Delta$A (norm.)', fontsize=fs)
    ax1.set_xlabel('Time (ps)', fontsize=fs)
   
fig.subplots_adjust(left=.09, bottom=.08, right=.98, top=.96)
fig.set_size_inches(double_col_width, double_col_width*0.86)
fig.savefig(os.path.join('fig2x.{0}'.format(fmt)), format=fmt, dpi=dpi)#, bbox_inches='tight')