import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import usefulfunctions as uf
from tripletpairs.toolkit import convolve_irf
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
import matplotlib.gridspec as gridspec


# plotting parameters
plt.rc('font', family='serif', serif='Arial')
single_col_width = 3.26772
double_col_width = 6.73228
fs = 8
lw = 1
ax_lw = 0.8
ax_tickl = 4
cut = 10
cmap = plt.cm.RdBu_r
ms = 2.5
cs = 2.5
cbar_ax_colour = 'k'
colour_rubrene = 'darkorange'
colour_dbp = 'darkred'
colour_triplet = 'darkblue'
dpi = 300
fmt = 'svg'

# TCSPC
tcspc = pd.read_csv(os.path.join('data', 'Rub_TCSPC.csv'), index_col=0, header=None, squeeze=True)
tcspc_irf = pd.read_csv(os.path.join('data', 'TCSPC_IRF.csv'), index_col=0, header=None, squeeze=True)

# integrated PL
intpl = pd.read_csv(os.path.join('data', 'Rub_integrated_PL.csv'), index_col=0, header=0)

# TA
ta = pd.read_csv(os.path.join('data', 'Rub_TA_510nm.csv'), header=0, index_col=0)

# Tdep PL data
temps = ['291K', '200K', '100K']
pl_tdep = {}
for temp in temps:
    df = pd.read_csv(os.path.join('data', 'Rub_TRPL_{0}_integrated.csv'.format(temp)), header=0, index_col=0)
    pl_tdep[temp] = df

# MFE
mfe = 100*pd.read_csv(os.path.join('data', 'Rub_MFE.csv'), header=0, index_col=0)


# create the grid
fig = plt.figure()

grid = gridspec.GridSpec(1, 2, wspace=0.4, width_ratios=[1, 1])

gridl = gridspec.GridSpecFromSubplotSpec(2, 1, subplot_spec=grid[0], hspace=0.25, height_ratios=[2, 1])
gridr = gridspec.GridSpecFromSubplotSpec(2, 1, subplot_spec=grid[1], hspace=0.22, height_ratios=[3, 1])

gridlt = gridspec.GridSpecFromSubplotSpec(2, 1, subplot_spec=gridl[0], hspace=0.07)
gridlb = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridl[1])
gridrt = gridspec.GridSpecFromSubplotSpec(3, 1, subplot_spec=gridr[0], hspace=0.1)
gridrb = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridr[1])

# TRPL
inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridlt[0])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

yt = tcspc
yt.index -= yt.idxmax()
yt = yt/yt.max()
ax.loglog(yt.index, yt, marker='o', linestyle='None', color='dodgerblue', ms=ms, mew=0, alpha=0.3)

irf_x = tcspc_irf.index-tcspc_irf.idxmax()
irf_y = tcspc_irf/tcspc_irf.max()
ax.loglog(irf_x[irf_x < 20], irf_y[irf_x < 20], 'k-', linewidth=lw, alpha=0.5)

ax.text(0.03, 0.75, 'IRF', fontsize=fs, color='0.5', transform=ax.transAxes)

trpl = pl_tdep['291K']
for i, pulse_energy in enumerate(trpl.columns):
    yi = trpl[pulse_energy]
    yi = yi[yi.index >= 8]
    yi *= 0.2 # 0.45 CPC
    ax.loglog(yi.index, yi, marker='o', linestyle='None', color=colour_rubrene, ms=ms, mew=0, label=pulse_energy.replace('u', r'$\mu$').replace('cm2', r'cm$^{-2}$'), alpha=(i+1)/3)

ax.legend(frameon=False, loc='upper right', borderpad=0, fontsize=fs, handletextpad=0, handlelength=1, labelspacing=0.08)

tx = yt.index.values
yp = np.exp(-tx/3)
tx, yp = convolve_irf(tx, yp, 0.7, shift_max_to_zero=True)
yp /= max(yp)

ax.loglog(tx, yp, linestyle='--', color='forestgreen', linewidth=lw)
ax.text(0.03, 0.2, r'exp$(-\frac{t}{3})$', fontsize=fs, color='forestgreen', transform=ax.transAxes)

ax.fill_between(tx, yp, color='forestgreen', linewidth=0, alpha=0.1)
ax.text(0.08, 0.05, '< 66%', fontsize=fs, color='forestgreen', transform=ax.transAxes)

td = yi.index.values
yd = 1.2*td**-1.3  # 2*td**-1.3

ax.loglog(td, yd, color='k', linestyle='--', linewidth=lw)
ax.text(0.5, 0.1, r'$t^{-1.3}$', fontsize=fs, color='k', transform=ax.transAxes)

ymin = 1e-5
ymax = 1.5
ax.set_ylim(ymin, ymax)
ax.set_xlim(0.2, 1e5)

ax.fill_between((100, 700), ymin, ymax, color='k', alpha=0.1, linewidth=0)

ax = uf.logticks(ax, which='both')
ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)

ax.set_ylabel('PL intensity (norm.)', fontsize=fs)
ax.set_xticklabels([])

y = pd.concat((yt[yt.index < 6], yi[yi.index >= 6]))

y = np.hstack((yp[tx < 6], yd[td >= 6]))
yidx = np.hstack((tx[tx < 6], td[td >= 6]))
y = pd.Series(index=yidx, data=y)

total = np.trapz(y, x=y.index.values)
prompt = np.trapz(yp, x=tx)


# TA
inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridlt[1])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

alphas = [0.2, 0.4, 0.8]
for p, pulse_energy in enumerate(ta.columns):
    
    ta_x = ta[pulse_energy]
    
    normf = ta_x[(ta_x.index > 5) & (ta_x.index < 10)].mean()
    scale = 0.9
    
    ta_xp = scale*ta_x/normf
    ta_xp.index = ta_xp.index+1-ta_xp.idxmax()
    
    ax.loglog(ta_xp.index, ta_xp, color=colour_triplet, linestyle='None', marker='o', ms=ms, mew=0, alpha=alphas[p], label=pulse_energy.replace('u', r'$\mu$').replace('cm2', r'cm$^{-2}$'))
    
ax.set_xlim(0.2, 1e5)
ymin = 7e-3
ymax = 1.2
ax.set_ylim(ymin, ymax)

ax = uf.logticks(ax, which='both')
ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)

ax.fill_between([100, 700], ymin, ymax, color='k', alpha=0.1, linewidth=0)

ax.set_ylabel(r'$\Delta$T/T (norm.)', fontsize=fs)
ax.set_xlabel('Time (ns)', fontsize=fs)

ax.legend(frameon=False, loc=(0.02, 0.02), borderpad=0, fontsize=fs, handletextpad=0, handlelength=1, labelspacing=0.08) 

    
# MFE
inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridlb[0])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

ax.axhline(0, color='k', alpha=0.5, linewidth=1)
ax.errorbar(mfe.index, mfe['20-30ns'], yerr=mfe['20-30ns err'], linestyle='None', color='dodgerblue', marker='x', ms=ms, capsize=cs, label='20ns')
ax.errorbar(mfe.index, mfe['200-300ns'], yerr=mfe['200-300ns err'], linestyle='None', color=colour_rubrene, marker='x', ms=ms, capsize=cs, label='200ns')

ax.set_ylabel(r'$\Delta$PL/PL (%)', fontsize=fs, labelpad=-2)
ax.set_xlabel('Magnetic field strength (mT)', fontsize=fs)

ax.set_ylim(-10, 65)
ax.set_xlim(-5, 250)

ax.legend(frameon=False, fontsize=fs, loc='upper left', handletextpad=0.1)

ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)
ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=10, yminorsep=5)


  
# T-dep PL
for i, temp in enumerate(temps):

    pl_x = pl_tdep[temp]
    
    inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridrt[i])
    ax = plt.Subplot(fig, inner[0])
    fig.add_subplot(ax)
    
    alphas = [0.333, 0.667, 1]
            
    if i == 2:
        x2 = np.logspace(1.5, 5)
        y2 = 5*x2**-1.3
        ax.loglog(x2, y2, color='black', linestyle='--', alpha=1, linewidth=lw)
        
        xexp = np.geomspace(1, 100, 100)
        ax.loglog(xexp, np.exp(-xexp/16), 'g--', linewidth=lw)
        ax.text(100, 0.2, r'exp$(-\frac{t}{16})$', fontsize=fs, color='g')
        ax.text(0.5, 0.25, r'$t^{-1.3}$', fontsize=fs, color='k', transform=ax.transAxes)
        
    if i == 1:
        x2 = np.logspace(0.7, 5)
        y2 = 5*x2**-1.3
        ax.loglog(x2, y2, color='black', linestyle='--', alpha=1, linewidth=lw)
        ax.text(0.5, 0.25, r'$t^{-1.3}$', fontsize=fs, color='k', transform=ax.transAxes)
        
    if i == 0:
        x2 = np.logspace(0, 5)
        y2 = 5*x2**-1.3
        ax.loglog(x2, y2, color='black', linestyle='--', alpha=1, linewidth=lw)
        ax.text(0.5, 0.25, r'$t^{-1.3}$', fontsize=fs, color='k', transform=ax.transAxes)
    
    for p, pulse_energy in enumerate(pl_x.columns):    
        ax.loglog(pl_x.index, pl_x[pulse_energy], color=colour_rubrene, linestyle='None', marker='o', ms=ms, mew=0, alpha=alphas[p], label=pulse_energy.replace('u', r'$\mu$').replace('cm2', r'cm$^{-2}$'))

        
    ax.set_xlim(1, 8e4)
    ax.set_ylim(5e-7, 3)
    
    ax = uf.logticks(ax, which='both')
    for label in ax.yaxis.get_ticklabels()[1::2]:
        label.set_visible(False)
    ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)


    ax.set_ylabel(r'PL intensity (norm.)', fontsize=fs)
    
    if i == 2:
        ax.set_xlabel('Time (ns)', fontsize=fs)
    else:
        ax.set_xticklabels([])
    
    ax.legend(frameon=False, loc=(0.02, 0.02), borderpad=0, fontsize=fs, handletextpad=0, handlelength=1, labelspacing=0.08)
    
    ax.text(0.98, 0.98, temp, fontsize=fs, transform=ax.transAxes, ha='right', va='top')
  
    
# MFE
inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridrb[0])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

rub_int = intpl[['Rub 77', 'Rub RT']].mean(axis=1)
rub_int_err = intpl[['Rub 77', 'Rub RT']].std(axis=1)/np.sqrt(2)

#ax.axhline(3, color='k', alpha=0.5, linewidth=1)
ax.text(220, 15, '293K\nPLQY = 3%', fontsize=fs, alpha=1)

#ax.axhline(77, color='k', alpha=0.5, linewidth=1)
ax.text(120, 65, '77K\nPLQY = 77%', fontsize=fs, alpha=1)

ax.errorbar(rub_int.index, rub_int, yerr=rub_int_err, linestyle='None', color=colour_rubrene, marker='x', ms=ms, capsize=cs)

ax.set_ylabel(r'interpolated PLQY (%)', fontsize=fs)
ax.set_xlabel('Temperature (K)', fontsize=fs)

ax.set_ylim(0, 85)
ax.set_xlim(70, 300)

ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)
ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=20, yminorsep=10)
    
    
fig.subplots_adjust(left=.18, bottom=.07, right=.85, top=.98)
fig.set_size_inches(double_col_width, double_col_width*0.9)
fig.savefig(os.path.join('fig3x.{0}'.format(fmt)), format=fmt, dpi=dpi)#, bbox_inches='tight')