import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import usefulfunctions as uf
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.legend_handler import HandlerTuple
import matplotlib.gridspec as gridspec


# plotting parameters
plt.rc('font', family='serif', serif='Arial')
single_col_width = 3.26772
double_col_width = 6.73228
fs = 8
lw = 1
ax_lw = 0.8
ax_tickl = 4
cut = 10
cmap = plt.cm.RdBu_r
ms = 2
cbar_ax_colour = 'k'
colour_rubrene = 'darkorange'
colour_dbp = 'darkred'
colour_triplet = 'darkblue'
dpi = 300
fmt = 'svg'


# ps TA
rubDBP_psTA = pd.read_csv(os.path.join('data', 'RubDBP_NP_TA_data.csv'), header=0, index_col=0)
rubDBP_psTA.columns = [float(x) for x in rubDBP_psTA.columns]

rubDBP_spectra = pd.read_csv(os.path.join('data', 'RubDBP_NP_TA_spectra.csv'), header=0, index_col=0)

ids = [1, 2, 3]
rub_psTA_dynamics = {}
rubdbp_psTA_dynamics = {}
for p in ids:
    rub_psTA_dynamics[p] = pd.read_csv(os.path.join('data', 'Rub_NP_{0}_TA_dynamics.csv'.format(p)), header=0, index_col=0)
    rubdbp_psTA_dynamics[p] = pd.read_csv(os.path.join('data', 'RubDBP_NP_{0}_TA_dynamics.csv'.format(p)), header=0, index_col=0)


# create the grid
fig = plt.figure(figsize=(14, 8))

grid = gridspec.GridSpec(1, 2, wspace=0.4, width_ratios=[1, 1])

gridl = gridspec.GridSpecFromSubplotSpec(2, 1, subplot_spec=grid[0], hspace=0.07, height_ratios=[1.4, 1])
gridr = gridspec.GridSpecFromSubplotSpec(2, 1, subplot_spec=grid[1], hspace=0.07, height_ratios=[1, 1])


# psTA
inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridl[0])
ax = plt.Subplot(fig, inner[0])

fig.add_subplot(ax)


ta_x = rubDBP_psTA
x = ta_x.index.values
y = ta_x.columns.values
X, Y = np.meshgrid(x, y)
Z = np.transpose(ta_x.values*1000)
zmax = np.nanmax(np.abs(Z))
levels = np.linspace(-zmax, zmax, 50)

cs = ax.contourf(X, Y, Z, levels, cmap=cmap, extend='both', zorder=-20)
for c in cs.collections:
    c.set_edgecolor('face')
ax.set_rasterization_zorder(-10)
ax.set_xlim([min(x), max(x)])
ax.set_ylim([0.1, 6000])

ax.set_yscale('log')

axins = inset_axes(ax, width='5%', height='25%', loc='lower right')
cbar = fig.colorbar(cs, cax=axins, orientation='vertical', ticks=[-np.floor(zmax), 0, np.floor(zmax)], extendrect=True)
axins.tick_params(color=cbar_ax_colour, labelcolor=cbar_ax_colour, labelsize=fs)
axins.yaxis.tick_left()
axins.text(1.2, 1.1, r'$\Delta$A (mOD)', fontsize=fs, color=cbar_ax_colour, ha='right', va='bottom', transform=axins.transAxes)

ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)

ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10)

ax = uf.logticks(ax, which='y')

ax.set_xticklabels([])

ax.set_ylabel('Time (ps)', fontsize=fs)



# spectra
inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridl[1])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

ax.axhline(0, linewidth=1, color='0.5')

sp_x = rubDBP_spectra

ax.plot(sp_x.index, sp_x['S']/sp_x['S'].max(), color=colour_rubrene, linewidth=lw, label='singlet')
ax.plot(sp_x.index, sp_x['T']/sp_x['T'].max(), color=colour_triplet, linewidth=lw, label='triplet')
ax.plot(sp_x.index, -sp_x['DBP']/sp_x['DBP'].min(), color=colour_dbp, linewidth=lw, label='DBP')

ax.set_xlim(sp_x.index.min(), sp_x.index.max())
ax.set_ylim(-1.1, 1.1)

ax.set_ylabel(r'$\Delta$A (norm.)', fontsize=fs, labelpad=0)
    
ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)
ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=0.5, yminorsep=0.25)

ax.set_xlabel('Wavelength (nm)', fontsize=fs)

ax.legend(frameon=False, fontsize=fs, borderpad=0, loc='upper right', handlelength=1.2, labelspacing=0.28)


# kinetics comparison
inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridr[0])
ax1 = plt.Subplot(fig, inner[0])

fig.add_subplot(ax1)


p = 2

kn_x = rub_psTA_dynamics[p]

normfs = kn_x['S'].max()
tscale = 1
ax1.plot(kn_x.index, kn_x['S']/normfs, color='k', linestyle='None', marker='s', mfc='None', ms=ms, mew=1, alpha=0.4, label='Rub')
ax1.plot(kn_x.index, tscale*kn_x['T']/normfs, color='k', linestyle='None', marker='s', mfc='None', ms=ms, mew=1, alpha=0.4)

kn_x = rubdbp_psTA_dynamics[p]

normfs = kn_x['S'].max()
tscale = 1
ax1.plot(kn_x.index, kn_x['S']/normfs, color=colour_rubrene, linestyle='None', marker='o', ms=ms, mew=0, alpha=0.8, label='Rub/DBP: Rub singlet')
ax1.plot(kn_x.index, tscale*kn_x['T']/normfs, color=colour_triplet, linestyle='None', marker='o', ms=ms, mew=0, alpha=0.8)


ax1.set_xlim([0.1, 6000])
ax1.set_ylim(-0.1, 1.1)
ax1.set_xscale('log')

ax1.axhline(0, linewidth=1, color='0.5')   

ax1 = uf.linearticks(ax1, ymajorsep=0.2, yminorsep=0.1)
ax1 = uf.logticks(ax1, which='x')

ax1 = uf.format_axes(ax1, labelsize=fs, linewidth=ax_lw, length=ax_tickl)

ax1.set_ylabel(r'$\Delta$A (norm.)', fontsize=fs)
ax1.set_xticklabels([])

ax1.legend(frameon=False, fontsize=fs, borderpad=0, loc='upper right', handletextpad=0, handlelength=1)


# kinetics
inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridr[1])
ax1 = plt.Subplot(fig, inner[0])

fig.add_subplot(ax1)
    
alphas = [0.333, 0.667, 1]
conv = 1000/(5000*np.pi*0.05**2)
powers = conv*np.array([0.2, 0.7, 1.6])
for i, p in enumerate(powers):
    kn_x = rubdbp_psTA_dynamics[i+1]

    normfs = kn_x['S'].max()
    dbpscale = 1.0
    ax1.plot(kn_x.index, kn_x['S']/normfs, color=colour_rubrene, linestyle='None', marker='o', ms=ms, mew=0, alpha=alphas[i], label=r'{0:.0f}$\mu$Jcm$^{{-2}}$'.format(p))
    ax1.plot(kn_x.index, tscale*kn_x['T']/normfs, color=colour_triplet, linestyle='None', marker='o', ms=ms, mew=0, alpha=alphas[i], label=r'{0:.0f}$\mu$Jcm$^{{-2}}$'.format(p))
    ax1.plot(kn_x.index, dbpscale*kn_x['C']/normfs, color=colour_dbp, linestyle='None', marker='^', ms=ms, mew=0, alpha=alphas[i], label=r'{0:.0f}$\mu$Jcm$^{{-2}}$'.format(p))

ax1.set_xlim([0.1, 6000])

ax1.set_ylim(-0.1, 1.1)

ax1.set_xscale('log')

ax1.axhline(0, linewidth=1, color='0.5')   

ax1 = uf.linearticks(ax1, ymajorsep=0.2, yminorsep=0.1)
ax1 = uf.logticks(ax1, which='x')

ax1 = uf.format_axes(ax1, labelsize=fs, linewidth=ax_lw, length=ax_tickl)

ax1.set_ylabel(r'$\Delta$A (norm.)', fontsize=fs)
ax1.set_xlabel('Time (ps)', fontsize=fs)

hs, ls = ax1.get_legend_handles_labels()
ax1.legend(handles=zip(hs[::3], hs[1::3], hs[2::3]), labels=ls[::3], fontsize=fs, numpoints=1, handler_map={tuple: HandlerTuple(ndivide=None)}, frameon=False, loc='upper right', borderpad=0, labelspacing=0.08, handletextpad=0.1, handlelength=1.3)

fig.subplots_adjust(left=.18, bottom=.12, right=.85, top=.98)
fig.set_size_inches(double_col_width, double_col_width*0.6)
fig.savefig(os.path.join('fig5x.{0}'.format(fmt)), format=fmt, dpi=dpi)#, bbox_inches='tight')