import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import usefulfunctions as uf
from tripletpairs.toolkit import convolve_irf
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.legend_handler import HandlerTuple
import matplotlib.gridspec as gridspec


# plotting parameters
plt.rc('font', family='serif', serif='Arial')
single_col_width = 3.26772
double_col_width = 6.73228
fs = 8
lw = 1
ax_lw = 0.8
ax_tickl = 4
cut = 10
cmap = plt.cm.RdBu_r
ms = 2.5
cs = 2.5
cbar_ax_colour = 'k'
colour_rubrene = 'darkorange'
colour_dbp = 'darkred'
colour_triplet = 'darkblue'
dpi = 300
fmt = 'svg'

# TCSPC
tcspc_rub = pd.read_csv(os.path.join('data', 'Rub_TCSPC.csv'), index_col=0, header=None, squeeze=True)
tcspc_dbp = pd.read_csv(os.path.join('data', 'RubDBP_TCSPC.csv'), index_col=0, header=None, squeeze=True)

# integrated PL
rub_intpl = pd.read_csv(os.path.join('data', 'Rub_integrated_PL.csv'), index_col=0, header=0)
rubdbp_intpl = pd.read_csv(os.path.join('data', 'RubDBP_integrated_PL.csv'), index_col=0, header=0)

# TA
ta_rub = pd.read_csv(os.path.join('data', 'Rub_TA_510nm.csv'), header=0, index_col=0)
ta_dbp = pd.read_csv(os.path.join('data', 'RubDBP_TA_510nm.csv'), header=0, index_col=0)

# TRPL
trpl_rub = pd.read_csv(os.path.join('data', 'Rub_NP_TRPL_integrated.csv'), header=0, index_col=0, squeeze=True)
trpl_dbp = pd.read_csv(os.path.join('data', 'RubDBP_NP_TRPL_integrated.csv'), header=0, index_col=0, squeeze=True)

# MFE
mfe_rub = 100*pd.read_csv(os.path.join('data', 'Rub_MFE.csv'), header=0, index_col=0)
mfe_dbp = 100*pd.read_csv(os.path.join('data', 'RubDBP_MFE.csv'), header=0, index_col=0)


# create the grid
fig = plt.figure()

grid = gridspec.GridSpec(1, 2, wspace=0.4, width_ratios=[1, 1])

gridl = gridspec.GridSpecFromSubplotSpec(2, 1, subplot_spec=grid[0], hspace=0.07, height_ratios=[1, 1])
gridr = gridspec.GridSpecFromSubplotSpec(2, 1, subplot_spec=grid[1], hspace=0.27, height_ratios=[1, 1])

# TRPL
inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridl[0])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

joinpoint = 30

yt = tcspc_rub
yt.index -= yt.idxmax()
yt = yt/yt.max()
ax.loglog(yt.index[yt.index <= joinpoint], yt[yt.index <= joinpoint], marker='o', linestyle='None', color='dodgerblue', ms=ms, mew=0, alpha=1, label='Rub')

yt = tcspc_rub
yt.index -= yt.idxmax()
yt = yt/yt.max()
ax.loglog(yt.index[yt.index > joinpoint], yt[yt.index > joinpoint], marker='o', linestyle='None', color='dodgerblue', ms=ms, mew=0, alpha=0.02, label='_nolegend_')

yi = trpl_rub
yi = yi[yi.index >= 10]
yi *= 1.2e-7
ax.loglog(yi.index, yi, marker='o', linestyle='None', color=colour_rubrene, ms=ms, mew=0, label='Rub')

yt = tcspc_dbp
yt.index -= yt.idxmax()
yt = yt/yt.max()
ax.loglog(yt.index[yt.index <= joinpoint], yt[yt.index <= joinpoint], marker='o', linestyle='None', color='teal', ms=ms, mew=0, alpha=1, label='Rub/DBP')

yt = tcspc_dbp
yt.index -= yt.idxmax()
yt = yt/yt.max()
ax.loglog(yt.index[yt.index > joinpoint], yt[yt.index > joinpoint], marker='o', linestyle='None', color='teal', ms=ms, mew=0, alpha=0.02, label='_nolegend_')

yi = trpl_dbp
yi = yi[yi.index >= 10]
yi *= 2e-5
ax.loglog(yi.index, yi, marker='o', linestyle='None', color=colour_dbp, ms=ms, mew=0, label='Rub/DBP')

pulse_energy = 5/(1000*np.pi*0.0025**2)
hs, ls = ax.get_legend_handles_labels()
l1 = ax.legend(handles=zip(hs[0::2], hs[1::2]), labels=ls[0::2], numpoints=1, handler_map={tuple: HandlerTuple(ndivide=None)}, frameon=False, loc='lower left', borderpad=0, fontsize=fs, handletextpad=0.1, handlelength=1.2, labelspacing=0.08)
for lh in l1.legendHandles: 
    lh._legmarker.set_alpha(1)
ax.add_artist(l1)
ax.legend(handles=zip([hs[1]], [hs[3]]), labels=[r'{0:.0f}$\mu$Jcm$^{{-2}}$'.format(pulse_energy)], numpoints=1, handler_map={tuple: HandlerTuple(ndivide=None)}, frameon=False, loc='upper right', borderpad=0, fontsize=fs, handletextpad=0.1, handlelength=1.2, labelspacing=0.08)

ax.set_ylim(1e-7, 1.5)
ax.set_xlim(0.2, 2e5)

ax = uf.logticks(ax, which='both')
ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)

ax.set_ylabel('PL intensity (norm.)', fontsize=fs)
ax.set_xticklabels([])


# TA
inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridl[1])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

idx = 1

ta_x = ta_rub.iloc[:, idx]
normf = ta_x[(ta_x.index > 5) & (ta_x.index < 10)].mean()
scale = 0.9
ta_xp = scale*ta_x/normf
ta_xp.index = ta_xp.index+1-ta_xp.idxmax()  
ax.loglog(ta_xp.index, ta_xp, color=colour_triplet, linestyle='None', marker='s', ms=ms, mew=0.5, mfc='None', label='Rub')

ta_x = ta_dbp.iloc[:, idx]
normf = ta_x[(ta_x.index > 5) & (ta_x.index < 10)].mean()
scale = 0.9
ta_xp = scale*ta_x/normf
#ta_xp.index = ta_xp.index+1-ta_xp.idxmax()  
ax.loglog(ta_xp.index, ta_xp, color=colour_triplet, linestyle='None', marker='o', ms=ms, mew=0, label='Rub/DBP')
    
ax.set_xlim(0.2, 2e5)
ax.set_ylim(7e-3, 1.2)

ax = uf.logticks(ax, which='both')
ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)

ax.text(0.98, 0.98, r'16$\mu$Jcm$^{-2}$', fontsize=fs, transform=ax.transAxes, ha='right', va='top')

ax.set_ylabel(r'$\Delta$T/T (norm.)', fontsize=fs)
ax.set_xlabel('Time (ns)', fontsize=fs)

ax.legend(frameon=False, loc=(0.02, 0.02), borderpad=0, fontsize=fs, handletextpad=0.1, handlelength=1, labelspacing=0.08) 

    
# MFE
inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridr[1])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

ax.axhline(0, color='k', alpha=0.5, linewidth=1)


ax.errorbar(mfe_rub.index, mfe_rub['20-30ns'], yerr=mfe_rub['20-30ns err'], linestyle='None', color='dodgerblue', marker='x', ms=ms, capsize=cs, label='Rub 20ns', alpha=0.3)
ax.errorbar(mfe_rub.index, mfe_rub['200-300ns'], yerr=mfe_rub['200-300ns err'], linestyle='None', color=colour_rubrene, marker='x', ms=ms, capsize=cs, label='Rub 200ns', alpha=0.3)

ax.errorbar(mfe_dbp.index, mfe_dbp['20-30ns'], yerr=mfe_dbp['20-30ns err'], linestyle='None', color='teal', marker='x', ms=ms, capsize=cs, label='Rub/DBP 20ns')
ax.errorbar(mfe_dbp.index, mfe_dbp['200-300ns'], yerr=mfe_dbp['200-300ns err'], linestyle='None', color=colour_dbp, marker='x', ms=ms, capsize=cs, label='Rub/DBP 200ns')

ax.set_ylabel(r'$\Delta$PL/PL (%)', fontsize=fs, labelpad=-2)
ax.set_xlabel('Magnetic field strength (mT)', fontsize=fs)

ax.set_ylim(-25, 65)
ax.set_xlim(-5, 250)

ax.legend(frameon=False, fontsize=fs, loc='upper left', handletextpad=0.1, borderpad=0)

ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)
ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=10, yminorsep=5)

   
# Tdep
inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridr[0])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

rub_int = rub_intpl[['Rub 77', 'Rub RT']].mean(axis=1)
rub_int_err = rub_intpl[['Rub 77', 'Rub RT']].std(axis=1)/np.sqrt(2)

dbp_int = rubdbp_intpl[['RubDBP 77', 'RubDBP RT']].mean(axis=1)
dbp_int_err = rubdbp_intpl[['RubDBP 77', 'RubDBP RT']].std(axis=1)/np.sqrt(2)

#ax.axhline(3, color='k', alpha=0.5, linewidth=1)
ax.text(210, 50, '293K\nPLQY = 61%', fontsize=fs, alpha=1)

#ax.axhline(77, color='k', alpha=0.5, linewidth=1)
ax.text(190, 85, '77K\nPLQY = 94%', fontsize=fs, alpha=1)

ax.errorbar(rub_int.index, rub_int, yerr=rub_int_err, linestyle='None', color=colour_rubrene, marker='x', ms=ms, capsize=cs, label='Rub')
ax.errorbar(dbp_int.index, dbp_int, yerr=dbp_int_err, linestyle='None', color=colour_dbp, marker='x', ms=ms, capsize=cs, label='Rub/DBP')

ax.set_ylabel(r'interpolated PLQY (%)', fontsize=fs)
ax.set_xlabel('Temperature (K)', fontsize=fs)

ax.set_ylim(0, 100)
ax.set_xlim(70, 300)

ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)
ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=20, yminorsep=10)

ax.legend(frameon=False, fontsize=fs, loc='lower left', handletextpad=0.1, borderpad=0) 
    
fig.subplots_adjust(left=.18, bottom=.12, right=.85, top=.98)
fig.set_size_inches(double_col_width, double_col_width*0.6)
fig.savefig(os.path.join('fig6x.{0}'.format(fmt)), format=fmt, dpi=dpi)#, bbox_inches='tight')