import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import usefulfunctions as uf
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.legend_handler import HandlerTuple
import matplotlib.gridspec as gridspec


# plotting parameters
plt.rc('font', family='serif', serif='Arial')
single_col_width = 3.26772
double_col_width = 6.73228
fs = 8
lw = 1
ax_lw = 0.8
ax_tickl = 4
cut = 10
cmap = plt.cm.RdBu_r
ms = 2
cbar_ax_colour = 'k'
colour_rubrene = 'darkorange'
colour_dbp = 'darkred'
colour_triplet = 'darkblue'
dpi = 300
fmt = 'svg'



# TA data
ta_x = pd.read_csv(os.path.join('data', 'RubDBP_film_TA_data.csv'), header=0, index_col=0)
ta_x.columns = [float(x) for x in ta_x.columns]

sp_x = pd.read_csv(os.path.join('data', 'Rub_film_TA_spectra.csv'), header=0, index_col=0)

kn_x = pd.read_csv(os.path.join('data', 'RubDBP_film_TA_dynamics.csv'), header=0, index_col=0)
kn_xi = pd.read_csv(os.path.join('data', 'Rub_film_TA_dynamics.csv'), header=0, index_col=0)

# PL comparison
pl = pd.read_csv(os.path.join('data', 'Rub_PL_compare.csv'), header=0, index_col=0)



# create the grid
fig = plt.figure()

grid = gridspec.GridSpec(1, 2, wspace=0.4)

gridrt = gridspec.GridSpecFromSubplotSpec(2, 1, subplot_spec=grid[0], hspace=0.07, height_ratios=[1.5, 1])
gridrb = gridspec.GridSpecFromSubplotSpec(2, 1, subplot_spec=grid[1], hspace=0.36)

# ps TA data
    
# colour maps

inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridrt[0])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

x = ta_x.index.values
y = ta_x.columns.values
X, Y = np.meshgrid(x, y)
Z = np.transpose(ta_x.values*1000)
zmax = np.nanmax(np.abs(Z))
levels = np.linspace(-zmax, zmax, 50)

cs = ax.contourf(X, Y, Z, levels, cmap=cmap, extend='both', zorder=-20)
for c in cs.collections:
    c.set_edgecolor('face')
ax.set_rasterization_zorder(-10)
ax.set_xlim([min(x), max(x)])
ax.set_ylim([0.1, max(y)])
ax.set_yscale('log')

axins = inset_axes(ax, width='5%', height='25%', loc='upper right')
cbar = fig.colorbar(cs, cax=axins, orientation='vertical', ticks=[-np.floor(zmax), 0, np.floor(zmax)], extendrect=True)
axins.tick_params(color=cbar_ax_colour, labelcolor=cbar_ax_colour, labelsize=fs)
axins.yaxis.tick_left()
axins.text(1.2, -0.1, r'$\Delta$A (mOD)', fontsize=fs, color=cbar_ax_colour, ha='right', va='top', transform=axins.transAxes)

ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)

ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10)

ax = uf.logticks(ax, which='y')
ax.set_xticklabels([])

ax.set_ylabel('Time (ps)', fontsize=fs)

# spectra

inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridrt[1])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

ax.axhline(0, linewidth=1, color='0.5')

ax.plot(sp_x.index, sp_x['S'], color=colour_rubrene, linewidth=lw, label='singlet', zorder=3)
ax.plot(sp_x.index, sp_x['T'], color=colour_triplet, linewidth=lw, label='triplet', zorder=2)
ax.plot(sp_x.index, 0.5*sp_x['C'], color=colour_dbp, linewidth=lw, label='DBP', zorder=1)

ax.set_xlim(sp_x.index.min(), sp_x.index.max())
ax.set_ylim(-0.55, 1.1)

ax.set_ylabel(r'$\Delta$A (norm.)', fontsize=fs)
    
ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)
ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=0.2, yminorsep=0.1)

ax.set_xlabel('Wavelength (nm)', fontsize=fs)

ax.legend(frameon=False, fontsize=fs, borderpad=0, loc='upper right', handlelength=1.5)

# kinetics

inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridrb[0])
ax1 = plt.Subplot(fig, inner[0])
fig.add_subplot(ax1)

normf = kn_x['S'].max()
normfi = kn_xi['S'].max()

ax1.plot(kn_xi.index, kn_xi['S']/normfi, color='k', linestyle='None', marker='s', mfc='None', ms=ms, mew=1, alpha=0.4, label='Rub only')

ax1.plot(kn_xi.index, kn_xi['T']/normfi, color='k', linestyle='None', marker='s', mfc='None', ms=ms, mew=1, alpha=0.4)

ax1.plot(kn_x.index, kn_x['S']/normf, color=colour_rubrene, linestyle='None', marker='o', ms=ms, mew=0, alpha=0.8)

ax1.plot(kn_x.index, kn_x['T']/normf, color=colour_triplet, linestyle='None', marker='o', ms=ms, mew=0, alpha=0.8)

ax1.plot(kn_x.index, kn_x['C']/normf, color=colour_dbp, linestyle='None', marker='o', ms=ms, mew=0, alpha=0.8)

ax1.legend(frameon=False, fontsize=fs, handlelength=1.3, handletextpad=0.3, loc=(0.4, 0.4), labelspacing=0.3)

    
ax1.set_xlim([0.1, kn_x.index.max()])
ax1.set_ylim(-0.1, 1.1)
ax1.set_xscale('log')

ax1.axhline(0, linewidth=1, color='0.5')   

ax1 = uf.linearticks(ax1, ymajorsep=0.2, yminorsep=0.1)
ax1 = uf.logticks(ax1, which='x')

ax1 = uf.format_axes(ax1, labelsize=fs, linewidth=ax_lw, length=ax_tickl)

ax1.set_ylabel(r'$\Delta$A (norm.)', fontsize=fs)
ax1.set_xlabel('Time (ps)', fontsize=fs, ha='right')


# PL spectra

inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gridrb[1])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

ax.plot(pl.index, pl['RUB_DBP_film']/10000, color=colour_dbp, linewidth=lw, label='Rub/DBP')
ax.plot(pl.index, pl['RUB_film']/10000, color=colour_rubrene, linewidth=lw, label='Rub only')

ax.set_xlim(525, 750)
ax.set_ylim(0, 1.75)

ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)
ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10, ymajorsep=0.5, yminorsep=0.25)

ax.legend(frameon=False, fontsize=fs, handlelength=1.3, handletextpad=0.3, loc='upper right', labelspacing=0.3)

ax.set_xlabel('Wavelength (nm)', fontsize=fs)
ax.set_ylabel(r'PL intensity (counts/10$^{4}$)', fontsize=fs)

   
fig.subplots_adjust(left=.18, bottom=.12, right=.85, top=.98)
fig.set_size_inches(double_col_width, double_col_width*0.6)
fig.savefig(os.path.join('figS11x.{0}'.format(fmt)), format=fmt, dpi=dpi)#, bbox_inches='tight')