import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import usefulfunctions as uf
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.legend_handler import HandlerTuple
import matplotlib.gridspec as gridspec


# plotting parameters
plt.rc('font', family='serif', serif='Arial')
single_col_width = 3.26772
double_col_width = 6.73228
fs = 8
lw = 1
ax_lw = 0.8
ax_tickl = 4
cut = 10
cmap = plt.cm.RdBu_r
ms = 2.5
cbar_ax_colour = 'k'
colour_rubrene = 'darkorange'
colour_dbp = 'darkred'
colour_triplet = 'darkblue'
dpi = 300
fmt = 'svg'



# load data
rub_stored = pd.read_csv(os.path.join('data', 'Rub_NP_532nm_stored.csv'), header=0, index_col=0)
rub_stored.columns = [float(x) for x in rub_stored.columns]

rub_fresh = pd.read_csv(os.path.join('data', 'Rub_NP_400nm_fresh.csv'), header=0, index_col=0)
rub_fresh.columns = [float(x) for x in rub_fresh.columns]

rubdbp_stored = pd.read_csv(os.path.join('data', 'RubDBP_NP_532nm_stored.csv'), header=0, index_col=0)
rubdbp_stored.columns = [float(x) for x in rubdbp_stored.columns]

rubdbp_fresh = pd.read_csv(os.path.join('data', 'RubDBP_NP_400nm_fresh.csv'), header=0, index_col=0)
rubdbp_fresh.columns = [float(x) for x in rubdbp_fresh.columns]





# create the grid
fig = plt.figure()

grid = gridspec.GridSpec(2, 3, hspace=0.36, wspace=0.32, width_ratios=[1, 1, 1.2])

    
# colour map Rub stored

ta_x = rub_stored

inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=grid[0, 0])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

x = ta_x.index.values
y = ta_x.columns.values
X, Y = np.meshgrid(x, y)
Z = np.transpose(ta_x.values*1000)
zmax = np.nanmax(np.abs(Z))
levels = np.linspace(-zmax, zmax, 50)

cs = ax.contourf(X, Y, Z, levels, cmap=cmap, extend='both', zorder=-20)
for c in cs.collections:
    c.set_edgecolor('face')
ax.set_rasterization_zorder(-10)
ax.set_xlim([min(x), max(x)])
ax.set_ylim([0.1, 5000])
ax.set_yscale('log')

axins = inset_axes(ax, width='5%', height='25%', loc='upper right')
cbar = fig.colorbar(cs, cax=axins, orientation='vertical', ticks=[-np.floor(zmax), 0, np.floor(zmax)], extendrect=True)
axins.tick_params(color=cbar_ax_colour, labelcolor=cbar_ax_colour, labelsize=fs)
axins.yaxis.tick_left()
axins.text(1.2, -0.1, r'$\Delta$A (mOD)', fontsize=fs, color=cbar_ax_colour, ha='right', va='top', transform=axins.transAxes)

ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)

ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10)
ax = uf.logticks(ax, which='y')

ax.set_xlabel('Wavelength (nm)', fontsize=fs)
ax.set_ylabel('Time (ps)', fontsize=fs)
ax.set_title('Rub, 532nm pump, 1yr storage', fontsize=fs)



# colour map Rub fresh

ta_x = rub_fresh

inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=grid[0, 1])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

x = ta_x.index.values
y = ta_x.columns.values
X, Y = np.meshgrid(x, y)
Z = np.transpose(ta_x.values*1000)
#zmax = np.nanmax(np.abs(Z))
zmax = 2
levels = np.linspace(-zmax, zmax, 50)

cs = ax.contourf(X, Y, Z, levels, cmap=cmap, extend='both', zorder=-20)
for c in cs.collections:
    c.set_edgecolor('face')
ax.set_rasterization_zorder(-10)
ax.set_xlim([min(x), max(x)])
ax.set_ylim([0.1, 5000])
ax.set_yscale('log')

axins = inset_axes(ax, width='5%', height='25%', loc='upper right')
cbar = fig.colorbar(cs, cax=axins, orientation='vertical', ticks=[-np.floor(zmax), 0, np.floor(zmax)], extendrect=True)
axins.tick_params(color=cbar_ax_colour, labelcolor=cbar_ax_colour, labelsize=fs)
axins.yaxis.tick_left()
axins.text(1.2, -0.1, r'$\Delta$A (mOD)', fontsize=fs, color=cbar_ax_colour, ha='right', va='top', transform=axins.transAxes)

ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)
ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10)
ax = uf.logticks(ax, which='y')

ax.set_xlabel('Wavelength (nm)', fontsize=fs)
ax.set_ylabel('Time (ps)', fontsize=fs)
ax.set_title('Rub, 400nm pump, freshly prepared', fontsize=fs)


    
# kinetics rub

wls = [440, 510]
dw = 2

colours = dict(zip(wls, ['k', 'r']))

inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=grid[0, 2])
ax1 = plt.Subplot(fig, inner[0])
fig.add_subplot(ax1)

norms = rub_stored[(rub_stored.index > wls[0]-dw) & (rub_stored.index < wls[0]+dw)].mean().max()
normf = rub_fresh[(rub_fresh.index > wls[0]-dw) & (rub_fresh.index < wls[0]+dw)].mean().max()

for wl in wls:
    
    kn_s = rub_stored[(rub_stored.index > wl-dw) & (rub_stored.index < wl+dw)].mean()
    kn_f = rub_fresh[(rub_fresh.index > wl-dw) & (rub_fresh.index < wl+dw)].mean()
    
    ax1.plot(kn_s.index, kn_s/norms, color=colours[wl], linewidth=lw, label='stored {0}nm'.format(wl))
    ax1.plot(kn_f.index, kn_f/normf, color=colours[wl], linestyle='None', marker='o', mfc='None', mew=lw, ms=ms, label='fresh {0}nm'.format(wl))
    
ax1.set_xlim([0.1, 5000])
ax1.set_ylim(-0.1, 1.1)
ax1.set_xscale('log')

ax1.axhline(0, linewidth=1, color='0.5')   

ax1 = uf.linearticks(ax1, ymajorsep=0.2, yminorsep=0.1)
ax1 = uf.logticks(ax1, which='x')

ax1 = uf.format_axes(ax1, labelsize=fs, linewidth=ax_lw, length=ax_tickl)

ax1.set_ylabel(r'$\Delta$A (norm.)', fontsize=fs)
ax1.set_xlabel('Time (ps)', fontsize=fs)

ax1.legend(frameon=False, fontsize=fs-1, loc=(0.55, 0.65), handlelength=1.2, handletextpad=0.5, borderpad=0)


# colour map RubDBP stored

ta_x = rubdbp_stored

inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=grid[1, 0])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

x = ta_x.index.values
y = ta_x.columns.values
X, Y = np.meshgrid(x, y)
Z = np.transpose(ta_x.values*1000)
zmax = np.nanmax(np.abs(Z))
levels = np.linspace(-zmax, zmax, 50)

cs = ax.contourf(X, Y, Z, levels, cmap=cmap, extend='both', zorder=-20)
for c in cs.collections:
    c.set_edgecolor('face')
ax.set_rasterization_zorder(-10)
ax.set_xlim([min(x), max(x)])
ax.set_ylim([0.1, 5000])
ax.set_yscale('log')

axins = inset_axes(ax, width='5%', height='25%', loc='upper right')
cbar = fig.colorbar(cs, cax=axins, orientation='vertical', ticks=[-np.floor(zmax), 0, np.floor(zmax)], extendrect=True)
axins.tick_params(color=cbar_ax_colour, labelcolor=cbar_ax_colour, labelsize=fs)
axins.yaxis.tick_left()
axins.text(1.2, -0.1, r'$\Delta$A (mOD)', fontsize=fs, color=cbar_ax_colour, ha='right', va='top', transform=axins.transAxes)

ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)

ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10)
ax = uf.logticks(ax, which='y')

ax.set_xlabel('Wavelength (nm)', fontsize=fs)
ax.set_ylabel('Time (ps)', fontsize=fs)
ax.set_title('Rub/DBP, 532nm pump, 1yr storage', fontsize=fs)



# colour map RubDBP fresh

ta_x = rubdbp_fresh

inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=grid[1, 1])
ax = plt.Subplot(fig, inner[0])
fig.add_subplot(ax)

x = ta_x.index.values
y = ta_x.columns.values
X, Y = np.meshgrid(x, y)
Z = np.transpose(ta_x.values*1000)
#zmax = np.nanmax(np.abs(Z))
zmax = 1
levels = np.linspace(-zmax, zmax, 50)

cs = ax.contourf(X, Y, Z, levels, cmap=cmap, extend='both', zorder=-20)
for c in cs.collections:
    c.set_edgecolor('face')
ax.set_rasterization_zorder(-10)
ax.set_xlim([min(x), max(x)])
ax.set_ylim([0.1, 5000])
ax.set_yscale('log')

axins = inset_axes(ax, width='5%', height='25%', loc='upper right')
cbar = fig.colorbar(cs, cax=axins, orientation='vertical', ticks=[-np.floor(zmax), 0, np.floor(zmax)], extendrect=True)
axins.tick_params(color=cbar_ax_colour, labelcolor=cbar_ax_colour, labelsize=fs)
axins.yaxis.tick_left()
axins.text(1.2, -0.1, r'$\Delta$A (mOD)', fontsize=fs, color=cbar_ax_colour, ha='right', va='top', transform=axins.transAxes)

ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)

ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10)

ax = uf.logticks(ax, which='y')

ax.set_xlabel('Wavelength (nm)', fontsize=fs)
ax.set_ylabel('Time (ps)', fontsize=fs)
ax.set_title('Rub/DBP, 400nm pump, freshly prepared', fontsize=fs)


    
# kinetics rub

wls = [440, 510, 600]
dw = 2

colours = dict(zip(wls, ['k', 'r', 'b']))

inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=grid[1, 2])
ax1 = plt.Subplot(fig, inner[0])
fig.add_subplot(ax1)

norms = rubdbp_stored[(rubdbp_stored.index > wls[0]-dw) & (rubdbp_stored.index < wls[0]+dw)].mean().max()
normf = rubdbp_fresh[(rubdbp_fresh.index > wls[0]-dw) & (rubdbp_fresh.index < wls[0]+dw)].mean().max()

for wl in wls:
    
    kn_s = rubdbp_stored[(rubdbp_stored.index > wl-dw) & (rubdbp_stored.index < wl+dw)].mean()
    kn_f = rubdbp_fresh[(rubdbp_fresh.index > wl-dw) & (rubdbp_fresh.index < wl+dw)].mean()
    
    ax1.plot(kn_s.index, kn_s/norms, color=colours[wl], linewidth=lw, label='stored {0}nm'.format(wl) if wl==wls[-1] else '_nolegend_')
    ax1.plot(kn_f.index, kn_f/normf, color=colours[wl], linestyle='None', marker='o', mfc='None', mew=lw, ms=ms, label='fresh {0}nm'.format(wl) if wl==wls[-1] else '_nolegend_')
    
ax1.set_xlim([0.1, 5000])
ax1.set_ylim(-0.6, 1.1)
ax1.set_xscale('log')

ax1.axhline(0, linewidth=1, color='0.5')   

ax1 = uf.linearticks(ax1, ymajorsep=0.2, yminorsep=0.1)
ax1 = uf.logticks(ax1, which='x')

ax1 = uf.format_axes(ax1, labelsize=fs, linewidth=ax_lw, length=ax_tickl)

ax1.set_ylabel(r'$\Delta$A (norm.)', fontsize=fs, labelpad=-2)
ax1.set_xlabel('Time (ps)', fontsize=fs)

ax1.legend(frameon=False, fontsize=fs-1, loc=(0.05, 0.02), handlelength=1.2, handletextpad=0.5, borderpad=0)
    

fig.subplots_adjust(left=.08, bottom=.10, right=.98, top=.94)
fig.set_size_inches(double_col_width, double_col_width*0.7)
fig.savefig(os.path.join('figS4x.{0}'.format(fmt)), format=fmt, dpi=dpi)#, bbox_inches='tight')