import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import usefulfunctions as uf
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.legend_handler import HandlerTuple
import matplotlib.gridspec as gridspec


# plotting parameters
plt.rc('font', family='serif', serif='Arial')
single_col_width = 3.26772
double_col_width = 6.73228
fs = 8
lw = 1
ax_lw = 0.8
ax_tickl = 4
cut = 10
cmap = plt.cm.RdBu_r
ms = 2
cbar_ax_colour = 'k'
colour_rubrene = 'darkorange'
colour_dbp = 'darkred'
colour_triplet = 'darkblue'
dpi = 600
fmt = 'png'



# load residuals
ids = [
    'Rub_np_1',
    'Rub_np_2',
    'Rub_np_3',
    'RubDBP_np_1',
    'RubDBP_np_2',
    'RubDBP_np_3',
    'Rub_f0.0',
    'Rub_f0.0',
    'Rub_f0.0'
]

tls = [
    'Rub NPs\n'+r'8$\mu$Jcm$^{{-2}}$',
    'Rub NPs\n'+r'18$\mu$Jcm$^{{-2}}$',
    'Rub NPs\n'+r'41$\mu$Jcm$^{{-2}}$',
    'Rub + 0.5mol% DBP NPs\n'+r'8$\mu$Jcm$^{{-2}}$',
    'Rub + 0.5mol% DBP NPs\n'+r'18$\mu$Jcm$^{{-2}}$',
    'Rub + 0.5mol% DBP NPs\n'+r'41$\mu$Jcm$^{{-2}}$',
    'Rub film\n'+r'41$\mu$Jcm$^{{-2}}$',
    'Rub film + 0.1mol% DBP\n'+r'41$\mu$Jcm$^{{-2}}$',
    'Rub film + 0.5mol% DBP\n'+r'41$\mu$Jcm$^{{-2}}$'
]

data = {}
for idx in ids:
    ta = pd.read_csv(os.path.join('data', '{0}.csv'.format(idx)), header=0, index_col=0)
    ta.columns = [float(x) for x in ta.columns]
    data[idx] = ta
    


# create the grid
fig = plt.figure()
grid = gridspec.GridSpec(3, 3, hspace=0.07, wspace=0.07)

mapping = dict(zip(range(9), [(0, 0), (1, 0), (2, 0), (0, 1), (1, 1), (2, 1), (0, 2), (1, 2), (2, 2)]))


# plot
for i, idx in enumerate(ids):
    
    row = mapping[i][0]
    col = mapping[i][1]
    
    ta_x = data[idx]

    inner = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=grid[row, col])
    ax = plt.Subplot(fig, inner[0])
    fig.add_subplot(ax)
    
    x = ta_x.index.values
    y = ta_x.columns.values
    X, Y = np.meshgrid(x, y)
    Z = np.transpose(ta_x.values*100)
    zmax = np.nanmax(np.abs(Z))
    zmax = 15
    levels = np.linspace(-zmax, zmax, 50)
    
    cs = ax.contourf(X, Y, Z, levels, cmap=cmap, extend='both', zorder=-20)
    for c in cs.collections:
        c.set_edgecolor('face')
    ax.set_rasterization_zorder(-10)
    ax.set_xlim([min(x), max(x)])
    ax.set_ylim(0.1, max(y))
    ax.set_yscale('log')
    
    axins = inset_axes(ax, width='5%', height='25%', loc='upper right')
    cbar = fig.colorbar(cs, cax=axins, orientation='vertical', ticks=[-np.floor(zmax), 0, np.floor(zmax)], extendrect=True)
    axins.tick_params(color=cbar_ax_colour, labelcolor=cbar_ax_colour, labelsize=fs)
    axins.yaxis.tick_left()
    axins.text(1.2, -0.1, r'$r / \Delta$A$_{max}$ (%)', fontsize=fs, color=cbar_ax_colour, ha='right', va='top', transform=axins.transAxes)

    ax = uf.format_axes(ax, labelsize=fs, linewidth=ax_lw, length=ax_tickl)
    
    ax = uf.linearticks(ax, xmajorsep=50, xminorsep=10)
    
    ax = uf.logticks(ax, which='y')
    
    if row < 2:
        ax.set_xticklabels([])
    else:
        ax.set_xlabel('Wavelength (nm)')
    
    if col > 0:
        ax.set_yticklabels([])
    else:
        ax.set_ylabel('Time (ps)', fontsize=fs)
        
    ax.text(0.03, 0.97, tls[i], fontsize=fs, transform=ax.transAxes, ha='left', va='top')

   
fig.subplots_adjust(left=.09, bottom=.08, right=.98, top=.98)
fig.set_size_inches(double_col_width, double_col_width)
fig.savefig(os.path.join('figS8.{0}'.format(fmt)), format=fmt, dpi=dpi)#, bbox_inches='tight')